#######################################################################
# Copyright (c) 2024 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

include IncidentsHelper

module Incidents
  class IncidentDruidRetriever
    def self.get_data(incident, options = {}, user)
      elements = get_incident_druid_data(incident, options, user)
      elements
    end

    def self.get_incident_druid_data(incident, search_params, user)
      data = []

      # Mapping of fields that can change depending on the module or data source
      field_map = {
        priority: [:priority, :syslogseverity_text], # Priority can be one of these fields
        msg: [:msg, :message],                       # Msg can be one of these fields
      }

      @objects = RedborderObject.where("user_id = ? OR user_id IS NULL", user.id)
      domain_ids = user.domains_with_permission.pluck(:id)
      incident_modules = Redborder::Module.registered_modules.select { |_, v| v[:columns] && v[:columns][:incident_uuid] }

      incident_modules.each do |mod|
        source = Redborder::DruidSource.get(source: mod.first)
        filter = Redborder::Filter.new(source, {})
        filter.add(:incident_uuid, incident.uuid)
        start_time = incident.created_at - 1.day
        end_time = incident.created_at + 1.day
        default_columns = source.default_columns_incidents

        data += source.where({
          query_type: :group_by,
          range: :custom,
          start_time: start_time,
          end_time: end_time,
          granularity: :minute,
          aggregation: source.default_aggregation,
          group_by: default_columns,
          search: filter.get,
          user: user
        }).map do |t|
          url_filters = { start_time: start_time, end_time: end_time }
          # Observables are define in the columns of each module
          observables = []
          assets = []
          observable_columns = default_columns
          # Filter columns by product type if the incident is from an alarm
          observable_columns = t.observable_columns if mod.first.to_s == "vault" && t.methods.include?(:alarm_product_type) && t.send(:alarm_product_type)

          mod.last[:columns].each do |column, cfg|
            next unless observable_columns.include?(column)
            next unless t.methods.include?(column)
            value = t.send(column)
            next if value.blank?

            url_filters[column] = value

            if cfg[:object_type] == "MacObject" || column.to_s == "asset_mac_address"
              mac_object = RedborderObject.where(sensor_id: domain_ids)
                                          .where.not(inventory_device_id: nil)
                                          .where(value: value, type: "MacObject")
                                          .first
              inventory_device = mac_object&.inventory_device
              if inventory_device
                if inventory_device.incidents.find_by(uuid: incident.uuid).nil?
                  inventory_device.incidents << incident
                  inventory_device.save
                end
                assets.push(inventory_device.to_asset(value))
              else
                observables.push({ value => cfg[:observable_type] || "mac_address" })
              end
            elsif column.to_s == "asset_ip_address"
              observables.push({ value => "ip" })
            elsif cfg[:observable_type]
              observables.push({ value => cfg[:observable_type] })
            end
          end

          # In case of vault alarm events
          # We try to extract observables/assets from the conditions of the alarm
          # Based on the product_type of the alarm
          # TODO: Refactor this thing
          if mod.first.to_s == "vault"

            alarm_product_type = nil
            if t.methods.include?(:alarm_product_type)
              alarm_product_type = t.send(:alarm_product_type) || nil
            end

            alarm_condition = nil
            if t.methods.include?(:alarm_condition)
              alarm_condition = t.send(:alarm_condition) || 'N/A'
            end

            columns = []
            if alarm_condition && alarm_product_type
              columns = IncidentDruidRetriever.get_columns_by_product_type(alarm_product_type)
            end

            unless columns.empty?
              alarm_condition_json = nil
              begin
                alarm_condition_json = JSON.parse(alarm_condition)
              rescue JSON::ParserError => e
                logger.error("Failed to parse alarm_condition JSON: #{e.message}")
                condition_json = nil
              end

              if alarm_condition_json
                condition_with_is_operator = alarm_condition_json.values.select { |item| item["operator"] == "is" }.map { |item| { "column" => item["column"], "value" => item["value"] } }

                columns.select { |column, cfg| cfg[:observable_type] }.each do |column, cfg|
                  # For each "is" condition
                  condition_with_is_operator.each do |c|
                    next unless column.to_s == c['column']

                    inventory_device = nil
                    if cfg[:object_type] == "MacObject"
                      mac_object = RedborderObject.where(sensor_id: user.domains_with_permission.pluck(:id))
                                          .where.not(inventory_device_id: nil)
                                          .where(value: c['value'], type: "MacObject")
                                          .first
                      inventory_device = mac_object.inventory_device if mac_object
                    end

                    if inventory_device
                      # Create relationship between incident and inventory device if it doesn't exist
                      if inventory_device.incidents.find_by(uuid: incident.uuid).nil?
                        inventory_device.incidents << incident
                        inventory_device.save
                      end

                      assets.push(inventory_device.to_asset(c['value']))
                    else
                      observables.push( { c['value'] => cfg[:observable_type] })
                    end
                  end
                end
              end
            end
          end

          url_filters["incident_uuid"] = incident.uuid

          {
            timestamp: t.send(:timestamp),
            priority: map_dynamic_fields(t, priority: field_map[:priority]),
            msg: map_dynamic_fields(t, msg: field_map[:msg]),
            incident_source_name: (mod.last[:incident_source_name] if mod.last.is_a?(Hash)) || "N/A",
            assets: assets,
            observables: observables,
            module_name: mod.first,
            url_filters: url_filters,
            sig_id: t.methods.include?(:sig_id) ? (t.send(:sig_id) || 'N/A') : 'N/A',
            classification: t.methods.include?(:classification) ? (t.send(:classification) || '') : '',
            src: t.methods.include?(:src) ? (t.send(:src) || 'N/A') : 'N/A',
            src_port: t.methods.include?(:src_port) ? (t.send(:src_port) || 'N/A') : 'N/A',
            dst: t.methods.include?(:dst) ? (t.send(:dst) || 'N/A') : 'N/A',
            dst_port: t.methods.include?(:dst_port) ? (t.send(:dst_port) || 'N/A') : 'N/A',
            ethsrc: t.methods.include?(:ethsrc) ? (t.send(:ethsrc) || 'N/A') : 'N/A',
            ethdst: t.methods.include?(:ethdst) ? (t.send(:ethdst) || 'N/A') : 'N/A',
            sensor_name: t.methods.include?(:sensor_name) ? (t.send(:sensor_name) || 'N/A') : 'N/A',
            sensor_uuid: t.methods.include?(:sensor_uuid) ? (t.send(:sensor_uuid) || 'N/A') : 'N/A'
          }
        end
      end

      # Filter by search
      if search_params && search_params[:source]
        data = data.select { |element| element[:incident_source_name] == search_params[:source] }
      end

      if search_params && search_params[:priority]
        data = data.select { |element| element[:priority] == search_params[:priority] }
      end

      data
    end

    def self.get_columns_by_product_type(product_type)
      incident_modules = Redborder::Module.registered_modules.select { |_, v| v[:columns] && v[:columns][:incident_uuid] }

      return [] unless incident_modules.include? product_type.to_sym

      return incident_modules[product_type.to_sym][:columns]
    end
  end
end